#include "gnu_getpass.h"

char *gnu_getpass(prompt,pass_buffer,bufsize)
     const char *prompt;
     char *pass_buffer;
     size_t bufsize;
{
  /* passing the buffer in so it can be " thread safe " */
  struct termios old, new; /* straight outta the info page */
  int bytes_read, fd, ctr, i;

  /* print the prompt */
  if(prompt != NULL)
  {
    write(fileno(stdout),prompt,strlen(prompt));
    fflush(stdout);
  }

  fd = fileno(stdin);

  if(tcgetattr(fd,&old) != 0)
  {
    fprintf(stderr,"Error getting terminal settings\r\ngoodbye.\r\n\a");
    exit(1); /* yes, exit()! */
  }

  new = old;
  new.c_lflag &= ~ECHO;

  if(tcsetattr(fd,TCSAFLUSH,&new) != 0)
  {
    fprintf(stderr,"Error setting terminal.\r\ngoodbye.\r\n\a");
    exit(1);
  }

  bytes_read = read(fd,pass_buffer,bufsize);

  /* remove any newlines/whitespace/etc */
  for(ctr=0;ctr<bytes_read;ctr++)
  {
    if( (pass_buffer[ctr] == '\r') ||
        (pass_buffer[ctr] == '\n') ||
        (pass_buffer[ctr] == '\t') ||
        (pass_buffer[ctr] == ' ') )
    {
      for(;ctr<bytes_read; ctr++)
      {
        /* clear out the remaining character(s) */
        pass_buffer[ctr] = 0;
      }
    }
  }

  /* restore the terminal */
  tcsetattr(fd, TCSAFLUSH, &old);

  return pass_buffer;
}
