
#include "ssl.h"
#include "common.h"

BIO *bio_err=0;

/* Print SSL errors and exit*/
int berr_exit(string)
     char *string;
{
  BIO_printf(bio_err,"%s\n",string);
  ERR_print_errors(bio_err);
  exit(0);
}

SSL_CTX *initialize_ctx(void)
{
  SSL_METHOD *meth;
  SSL_CTX *ctx;
  
  if(!bio_err)
  {
    /* Global system initialization*/
    SSL_library_init();
    SSL_load_error_strings();
    
    /* An error write context */
    bio_err=BIO_new_fp(stderr,BIO_NOCLOSE);
  }
    /* Create our context*/
  meth=TLSv1_client_method();
  ctx=SSL_CTX_new(meth);
  
  return ctx;
}

int SSL_receive(ssl, buf, len)
     SSL *ssl;
     char *buf;
     int len;
{
  char recv_buf[len];
  int r, bytes_received=0;
  
  buf[0]=0x0;
  while(bytes_received<len)
  {
    memset(recv_buf, 0x0, len);
    r = SSL_read(ssl,recv_buf,len);
    if(r<0) return -1;
    if(r==0) return 0;
    
    bytes_received+=r;
    strncat(buf, recv_buf, len);
  }
  return 0;
}


int get_https(host, port, request, buf, len)
     char *host;
     int port;
     char *request;
     char *buf;
     int len;
{
  SSL_CTX *ctx;
  SSL *ssl;
  BIO *sbio;
  int ret, sock;
  
  /* Build our SSL context*/
  ctx=initialize_ctx();
  
  /* Connect the TCP socket*/
  sock=tcp_connect(host,port);
  
  /* Connect the SSL socket */
  ssl=SSL_new(ctx);
  sbio=BIO_new_socket(sock,BIO_NOCLOSE);
  SSL_set_bio(ssl,sbio,sbio);
  
  if(SSL_connect(ssl)<=0)
    berr_exit("SSL connect error");
  
  if(SSL_write(ssl,request,strlen(request))<1)
    berr_exit("SSL write problem");
  
  if(SSL_receive(ssl,buf,BUF_SIZE)<0)
    berr_exit("SSL read problem");            
  
  if(SSL_shutdown(ssl)<0)
    berr_exit("SSL shutdown problem");
  
  /*** If the document has moved ***/
  if(strncmp(buf, "HTTP/1.1 302", 12)==0)
    ret = -2;
  /*** If the server returned an error ***/
  else if(strncmp(buf, "HTTP/1.1 200", 12)!=0)
    ret = -1;
  else ret = 0;
  
  SSL_free(ssl);
  SSL_CTX_free(ctx);
  close(sock);
  
  return(ret);
}

int https_auth(host,port,file,msn_login,msn_passwd,id_string,ticket,ticket_len)
     char *host;
     int port;
     char *file;
     char *msn_login;
     char *msn_passwd;
     char *id_string;
     char *ticket;
     int ticket_len;
{
   int ret, i, retry=0;
   char buf[BUF_SIZE];
   char host_buf[BUF_SIZE];
   char request[512], *ptr[2];
   
   strncpy(host_buf, host, sizeof(host_buf)-1);
   
   snprintf(request,sizeof(request)-1,
            "GET /%s HTTP/1.1\r\nAuthorization: "
            "Passport1.4 OrgVerb=GET,"
            "OrgURL=http://messenger.msn.com,"
            "sign-in=%s,pwd=%s,%s\r\nHost: "
            "%s:%d\r\n\r\n",
            file, 
            msn_login, 
            msn_passwd, 
            id_string,
            host_buf,port);
   
   /*** Authenticate ***/
   //status("Authenticating with server");
   
   /* If document has moved */
   while((ret=get_https(host_buf, port, request, buf, sizeof(buf)))==-2)
   {
     if((ptr[0] = strstr(buf, "Location:"))==NULL)
       err_exit("No 'Location:' in HTTP header");
     
     ptr[0][strlen(ptr[0])-4]=0x0;
     if((ptr[1]=(char *)split(ptr[0], '/', 2))==NULL)
       err_exit("Couldn't parse URL");
     
     if((ptr[2]=(char *)split(ptr[0], '/', 3))==NULL)
       err_exit("Couldn't parse URL");
     
     snprintf(request,sizeof(request)-1,
              "GET /%s HTTP/1.1\r\nAuthorization: "
              "Passport1.4 OrgVerb=GET,"
              "OrgURL=http://messenger.msn.com,"
              "sign-in=%s,pwd=%s,%s\r\nHost: "
              "%s:%d\r\n\r\n",
              ptr[2], 
              msn_login, 
              msn_passwd, 
              id_string,
              ptr[1],
              port);

     strncpy(host_buf, ptr[1], sizeof(host_buf)-1);
     //  for(i=1;i<3;i++) free(ptr[i]);
   }
   
   if(ret==-1)
     err_exit("Bad username or password!");
   
   //status("Extracting Passport-ticket");
   if((ptr[1]=(char *)split(buf, '\'',1))==NULL)
     err_exit("Couldn't extract ticket");
   
   strncpy(ticket, ptr[1], ticket_len);
   free(ptr[1]);
   return 0;
} 

